﻿using System;
using System.Linq;
using System.Security.AccessControl;
using System.Security.Principal;
using Alphaleonis.Win32.Filesystem;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AlphaFS.UnitTest
{
   // Bug with inherited folder rights.
   // https://alphafs.codeplex.com/workitem/19788

   //[TestClass]
   public class AlphaFSTests
   {
      private static readonly string SysDrive = Environment.GetEnvironmentVariable("SystemDrive");
      private static readonly string TestDir = SysDrive + @"\_AlphaFS-Sometest";

      [TestMethod]
      public void AlphaFS_SetSecurity_ShouldNotProduceError()
      {
         SetSecurityAlpha();
         //var dirsec = new System.IO.DirectoryInfo(@"e:\wifimtest\inherited").GetAccessControl();
         //var dirsecAlpha = _testDirInfoInherited.GetAccessControl(AccessControlSections.Access);
         //AuthorizationRuleCollection accessRules = dirsec.GetAccessRules(true, true, targetType: typeof(SecurityIdentifier));
         //AuthorizationRuleCollection accessRulesAlpha = dirsecAlpha.GetAccessRules(true, true, targetType: typeof(SecurityIdentifier));

         //Assert.IsFalse(HasLocalAccess(accessRules), "problem with system.");
         //Assert.IsFalse(HasLocalAccess(accessRulesAlpha), "problem with Alpha.");
      }

      [TestMethod]
      public void System_SetSecurity_ShouldNotProduceError()
      {
         SetSecuritySystem();
      }

      [TestMethod]
      public void System_AnalyzeSecurity_LocalAcesShouldNotExist()
      {
         SetSecuritySystem();
         DirectorySecurity dirsec = new System.IO.DirectoryInfo(TestDir + @"\inherited").GetAccessControl();
         AuthorizationRuleCollection accessRules = dirsec.GetAccessRules(true, true, targetType: typeof (SecurityIdentifier));

         bool has = HasLocalAccess(accessRules);
         Console.WriteLine("HasLocalAccess(accessRules): [{0}] == False = {1}", has, !has);
         Assert.IsFalse(has, "local access rules found.");
      }

      [TestMethod]
      public void AlphaFs_AnalyzeSecurity_LocalAcesShouldNotExist()
      {
         SetSecuritySystem();
         DirectorySecurity dirsec = new DirectoryInfo(TestDir + @"\inherited").GetAccessControl();
         AuthorizationRuleCollection accessRules = dirsec.GetAccessRules(true, true, targetType: typeof (SecurityIdentifier));
         
         bool has = HasLocalAccess(accessRules);
         Console.WriteLine("HasLocalAccess(accessRules): [{0}] == False = {1}", has, !has);
         Assert.IsFalse(HasLocalAccess(accessRules), "local access rules found.");
      }


      public static bool HasLocalAccess(AuthorizationRuleCollection rules)
      {
         AccessRule localAce = rules.Cast<AccessRule>().FirstOrDefault(a => a.IsInherited == false);
         bool res = (localAce != null);
         return res;
      }

      private static void SetSecurityAlpha()
      {
         //create the test structure
         if (Directory.Exists(TestDir))
            Directory.Delete(TestDir, true);

         Directory.CreateDirectory(TestDir);
         Directory.CreateDirectory(Path.Combine(TestDir, "inherited"));
         
         DirectoryInfo testDirInfo = new DirectoryInfo(TestDir);
         System.IO.DirectoryInfo testDirInfoSysIo = new System.IO.DirectoryInfo(TestDir);

         DirectorySecurity ds = testDirInfo.GetAccessControl(AccessControlSections.Access);
         DirectorySecurity dsSysIo = testDirInfoSysIo.GetAccessControl(AccessControlSections.Access);
         Assert.AreEqual(ds.AccessRightType, dsSysIo.AccessRightType);
         Assert.AreEqual(ds.AccessRuleType, dsSysIo.AccessRuleType);
         Assert.AreEqual(ds.AreAccessRulesCanonical, dsSysIo.AreAccessRulesCanonical);
         Assert.AreEqual(ds.AreAccessRulesProtected, dsSysIo.AreAccessRulesProtected);
         Assert.AreEqual(ds.AreAuditRulesCanonical, dsSysIo.AreAuditRulesCanonical);
         Assert.AreEqual(ds.AreAuditRulesProtected, dsSysIo.AreAuditRulesProtected);
         Assert.AreEqual(ds.AuditRuleType, dsSysIo.AuditRuleType);

         ds.SetAccessRuleProtection(true, false);
         ds.AddAccessRule(new FileSystemAccessRule(
                             identity: new SecurityIdentifier(WellKnownSidType.WorldSid, null),
                             fileSystemRights: FileSystemRights.FullControl,
                             inheritanceFlags: InheritanceFlags.ContainerInherit | InheritanceFlags.ObjectInherit,
                             propagationFlags: PropagationFlags.None,
                             type: AccessControlType.Allow
                             ));
         testDirInfo.SetAccessControl(ds);
      }

      private static void SetSecuritySystem()
      {
         //create the test structure
         if (System.IO.Directory.Exists(TestDir))
            System.IO.Directory.Delete(TestDir, true);

         System.IO.Directory.CreateDirectory(TestDir);
         System.IO.Directory.CreateDirectory(System.IO.Path.Combine(TestDir, "inherited"));
         System.IO.DirectoryInfo testDirInfo = new System.IO.DirectoryInfo(TestDir);

         DirectorySecurity ds = testDirInfo.GetAccessControl(AccessControlSections.Access);
         ds.SetAccessRuleProtection(true, false);
         ds.AddAccessRule(new FileSystemAccessRule(
                             identity: new SecurityIdentifier(WellKnownSidType.WorldSid, null),
                             fileSystemRights: FileSystemRights.FullControl,
                             inheritanceFlags: InheritanceFlags.ContainerInherit | InheritanceFlags.ObjectInherit,
                             propagationFlags: PropagationFlags.None,
                             type: AccessControlType.Allow
                             ));
         testDirInfo.SetAccessControl(ds);
      }

   }
}
